from datetime import datetime
from typing import Optional, List, Union
from pydantic import BaseModel, Field, ConfigDict
import enum


class TaskBase(BaseModel):
    """Base schema for tasks"""

    id: str
    title: str
    image_url: Optional[str] = None
    description: Optional[str] = None
    status: str
    priority: str
    due_date: Optional[datetime] = None
    created_at: datetime
    updated_at: datetime
    budget: float
    currency: str
    participants: int
    user_id: Optional[str] = None

    @staticmethod
    def format_date(value: datetime):
        return value.strftime("%b %d, %Y") if value else None
    
    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)
        data["created_at"] = self.format_date(self.created_at)
        data["updated_at"] = self.format_date(self.updated_at)
        data["due_date"] = self.format_date(self.due_date)

        return data

    model_config = ConfigDict(from_attributes=True)

class TaskCreate(BaseModel):
    """Schema to create a new task"""

    title: str = Field(..., example="Write Blog Post")
    description: Optional[str] = Field(None, example="Draft a blog post for the marketing team")
    image_url: Optional[str]
    status: Optional[str] = Field("Pending", example="Pending")
    priority: Optional[str] = Field("Medium", example="Medium")
    due_date: Optional[datetime] = Field(None, example="2024-12-25T23:59:59Z")
    required_user_role: Optional[str] = Field(None, example="Reply Guy")
    budget: float = Field(0.0)
    currency: str
    user_id:str


class TaskUpdate(BaseModel):
    """Schema to update a task"""

    title: Optional[str] = Field(None)
    description: Optional[str] = Field(None)
    status: Optional[str] = Field(None, example="Completed")
    priority: Optional[str] = Field(None, example="High")
    due_date: Optional[datetime] = Field(None)
    required_user_role: Optional[str] = Field(None, example="Reply Guy")
    budget: Optional[float]

class TaskResponse(BaseModel):
    """Schema to return task data"""

    id: str
    title: str
    description: Optional[str]
    status: str
    priority: str
    due_date: Optional[datetime]
    created_at: datetime
    updated_at: datetime
    assigned_user_id: Optional[str]
    required_user_role: Optional[str]
    budget: Optional[float]
    currency: str
    participants: int

    model_config = ConfigDict(from_attributes=True)

class AllTasksResponse(BaseModel):
    """Schema for returning all tasks"""

    message: str
    status_code: int
    status: str
    page: int
    per_page: int
    total: int
    data: Union[List[TaskResponse], List[None]]
